<?php
/**
 * @package Desk_Net_Request_Methods.
 * @author  Desk-Net GmbH.
 */

include_once plugin_dir_path( __FILE__ ) . 'class-desk-net-core.php';
include_once plugin_dir_path( __FILE__ ) . 'class-desk-net-delete-methods.php';
include_once plugin_dir_path( __FILE__ ) . 'class-desk-net-plugin-settings.php';
include_once plugin_dir_path( __FILE__ ) . 'class-desk-net-update-methods.php';

class Desk_Net_Request_Methods extends Desk_Net_Core {
	/**
	 * @var string Login.
	 */
	protected $login;

	/**
	 * @var string Password.
	 */
	protected $password;

	/**
	 * @var array Update Data.
	 */
	protected $update_data;

	/**
	 * Class constructor.
	 *
	 * @throws Exception.
	 */
	function __construct() {
		if ( ! function_exists( 'curl_init' ) ) {
			throw new Exception( 'CURL module not available! Pest requires CURL. See http://php.net/manual/en/book.curl.php' );
		}

		$this->login    = get_option( 'wp_desk_net_user_login' );
		$this->password = get_option( 'wp_desk_net_user_password' );

		add_action( 'admin_notices', array( $this, 'admin_notices' ) );
		add_action( 'rest_after_insert_post',
			array( $this, 'rest_api_post_update' ), 10, 3 );

		if ( ! empty( $this->login ) && ! empty( $this->password ) ) {
			if ( ! empty( get_option( 'wp_desk_net_token' ) )
			     && get_option( 'wp_desk_net_token' ) !== 'not_valid'
			) {
				$this->token = get_option( 'wp_desk_net_token' );
			} else {
				$this->token = $this->desk_net_get_token( $this->login,
					$this->password );
			}
			if ( false !== $this->token
			     && ! empty( get_option( 'wp_desk_net_platform_id' ) )
			) {
				add_action( 'load-post.php', array(
					$this,
					'edit_post_page',
				), 7 );
				add_action( 'load-edit.php', array(
					$this,
					'load_post_page',
				), 7 );
			} else {
				$this->error_message = 5;
			}
		} else {
			$this->error_message = 5;

			$this->token = false;
		}

		add_action( 'wp_print_scripts', array( $this, 'disable_autosave' ) );
		add_filter( 'redirect_post_location', array(
			$this,
			'add_notice_query_var',
		), 99 );
	}

	/**
	 * Detecting loading post page.
	 */
	public function load_post_page() {
		add_action( 'post_updated', array(
			$this,
			'desk_net_update_element',
		), 9, 3 );
	}

	/**
	 * Detecting editing post page.
	 */
	public function edit_post_page() {
		// Setting category to post revision.
		// @codingStandardsIgnoreStart.
		if ( isset( $_GET['post'] ) && ! empty( $_GET['post'] ) ) {
			$post_id = sanitize_text_field( wp_unslash( $_GET['post'] ) );
			// @codingStandardsIgnoreEnd.
			// Getting first active category.
			$active_category = get_the_terms( $post_id, 'category' );
			if ( ! empty( $active_category ) ) {
				// Setting active post category to post revision.
				update_post_meta( $post_id,
					'wp_desk_net_selected_categories_revision',
					get_the_terms( $post_id, 'category' ) );
				update_post_meta( $post_id, 'wp_desk_net_category_revision',
					array_values( $active_category )[0]->term_id );
			}
		}
		add_action( 'post_updated', array(
			$this,
			'desk_net_update_element',
		), 9, 3 );
	}

	/**
	 * Called when a Post has been Published via the REST API
	 *
	 * @param WP_Post         $post     Post
	 * @param WP_REST_Request $request  Request Object
	 * @param bool            $creating Post creating
	 *
	 * @since   1.0.0
	 *
	 */
	public function rest_api_post_update( $post, $request, $creating ) {
		$this->desk_net_update_element( $post->ID, $post );
	}

	/**
	 * Perform Disable AutoSave Draft.
	 */
	public function disable_autosave() {
		wp_deregister_script( 'autosave' );
	}

	/**
	 * Perform Update/Create Element on Desk-Net.
	 *
	 * @param integer $post_id
	 *     The post id in WP.
	 * @param object  $post_after
	 *     The post after update in WP.
	 * @param object  $post_before
	 *     The post before update in WP.
	 */
	public function desk_net_update_element(
		$post_id, $post_after, $post_before = null
	) {
		if ( 'post' === $post_after->post_type ) {
			// Getting Desk-Net revision.
			$story_removed_status = get_post_meta( $post_id,
				'wp_desk_net_remove_status' );
			$publication_id       = get_post_meta( $post_id, 'publications_id',
				true );
			$story_id             = get_post_meta( $post_id, 'story_id', true );

			if ( 'trash' === $post_after->post_status ) {
				if ( empty( $story_removed_status )
				     || 'removed' !== $story_removed_status[0]
				) {
					if ( $story_id ) {
						// Setting post status - 'Desk-Net Deleted/Removed'.
						update_post_meta( $post_id, 'wp_desk_net_remove_status',
							'removed' );

						// Check exist story on Desk-Net side.
						$story_data
							= json_decode( $this->desk_net_get( Desk_Net_Plugin_Settings::DN_BASE_URL,
							'elements', $story_id ), true );
						if ( ! empty( $story_data )
						     && empty( $story_data['message'] )
						) {
							$this->desk_net_custom_request( null,
								Desk_Net_Plugin_Settings::DN_BASE_URL,
								'elements/publication', 'DELETE',
								$publication_id );
						}
					}
				}

				return;

			} elseif ( ! empty( $post_before ) ) {
				if ( $post_after->post_content
				     !== $post_before->post_content
				) {
					update_post_meta( $post_id,
						'wp_desk_net_lock_update_post_content', true );
				}
				$post_after_time  = strtotime( $post_after->post_date_gmt );
				$post_before_time = strtotime( $post_before->post_date_gmt );

				if ( ( '0000-00-00 00:00:00' !== $post_after_time
				       && '0000-00-00 00:00:00' === $post_before_time )
				     || ( '0000-00-00 00:00:00' !== $post_before_time
				          && $post_after_time !== $post_before_time )
				) {
					update_post_meta( $post_id, 'wp_desk_net_change_post_date',
						'user_changed_post_date' );
				}
			}

			( new Desk_Net_Update_Methods() )->desk_net_update_story( $post_id,
				$post_after, $post_before );
		} else {
			remove_filter( 'redirect_post_location', array(
				$this,
				'add_notice_query_var',
			), 99 );
		}
	}

	/**
	 * Perform update message in admin panel
	 *
	 * @param integer        $post_id
	 *     The post id in WP.
	 * @param string|integer $message_id
	 *     The Desk-Net custom notice ID.
	 */
	public function set_desk_net_post_notice( $post_id, $message_id ) {
		update_post_meta( $post_id, 'desk_net_notice_message_id', $message_id );
	}

	/**
	 * Perform update message in admin panel
	 * param message_id equal zero Hiding admin Notice.
	 */
	public function admin_notices() {
		// @codingStandardsIgnoreStart.
		if ( ! isset( $_GET['message_id'] )
		     || 0 == sanitize_text_field( wp_unslash( $_GET['message_id'] ) )
		) {
			return;
		}
		// @codingStandardsIgnoreEnd.
		$message_list = array(
			1 => array(
				'line_1' => array(
					'text' => __( 'Story update successfully sent to Desk-Net',
						'desk_net' ),
				),
			),
			2 => array(
				'line_1' => array(
					'text' => __( 'Cannot update story in Desk-Net. The WordPress plugin could not find a corresponding story ID in Desk-Net. Code: 01',
						'desk_net' ),
				),
			),
			3 => array(
				'line_1' => array(
					'text' => __( 'Cannot update story in Desk-Net. There is no corresponding story in Desk-Net. Code: 02',
						'desk_net' ),
				),
			),
			4 => array(
				'line_1' => array(
					'text' => __( 'Cannot update the story in Desk-Net. Reason unknown. Please contact Desk-Net support at ',
						'desk_net' ),
					'link' => array(
						'url'  => 'mailto:support@desk-net.com',
						'text' => 'support@desk-net.com',
					),
				),
				'line_2' => array(
					'text' => '. Code: 03',
				),
			),
			5 => array(
				'line_1' => array(
					'text' => __( 'The Desk-Net API login credentials are not valid or have not been entered. Please check the settings on the page ',
						'desk_net' ),
					'link' => array(
						'url'  => '/wp-admin/admin.php?page=wp_desk_net_credential',
						'text' => __( 'Desk-Net Credentials', 'desk_net' ),
					),
				),
				'line_2' => array(
					'text' => __( ' in the Desk-Net plugin. Code: 04',
						'desk_net' ),
				),
			),
			6 => array(
				'line_1' => array(
					'text' => __( 'Cannot create story in Desk-Net. Reason unknown. Code: 05',
						'desk_net' ),
				),
			),
			7 => array(
				'line_1' => array(
					'text' => __( 'Cannot create story in Desk-Net. Publication date doesn\'t match platform schedule.',
						'desk_net' ),
				),
			),
			8 => array(
				'line_1' => array(
					'text' => __( 'Cannot update story in Desk-Net. Publication date doesn\'t match platform schedule.',
						'desk_net' ),
				),
			),
		);

		$warnings = array(
			1 => array(
				'line_1' => array(
					'text' => __( 'Not found this user in Desk-net',
						'desk_net' ),
				),
			),
		);
		// @codingStandardsIgnoreStart.
		if ( Desk_Net_Request_Methods::element_is_exist( $_GET['warning_id'] ) ) {
			$notice_id = explode( '_',
				sanitize_text_field( wp_unslash( $_GET['warning_id'] ) ) );

			Desk_Net_Request_Methods::outputting_system_notices( $warnings[ $notice_id ],
				'notice-warning' );
		}

		$notice_id = sanitize_text_field( wp_unslash( $_GET['message_id'] ) );

		if ( 3 > $notice_id ) {
			$type_notice = 'updated';
		} else {
			$type_notice = 'notice-error';
		}

		Desk_Net_Request_Methods::outputting_system_notices( $message_list[ $notice_id ],
			$type_notice );
		// @codingStandardsIgnoreEnd.
	}

	/**
	 * Printing the Desk-Net notices.
	 *
	 * @param array  $notice_content
	 *     The notice text with links.
	 * @param string $type_notice
	 *     The notice type.
	 */
	public static function outputting_system_notices(
		array $notice_content, $type_notice
	) {
		// Printing notice.
		?>
        <div
                class="<?php echo esc_attr( $type_notice ); ?> notice is-dismissible">
            <p>
				<?php
				foreach ( $notice_content as $content ) {
					echo esc_html( $content['text'] );

					if ( isset( $content['link'] ) ) {
						?>
                        <a href="<?php echo esc_url( $content['link']['url'] ); ?>"><?php echo esc_html( $content['link']['text'] ); ?></a>
						<?php
					}
				}
				?>
            </p>
        </div>
		<?php
	}

	/**
	 * Element check on existence.
	 *
	 * @param boolean|integer|string|array $element
	 *     The checking element.
	 *
	 * @return boolean
	 */
	public static function element_is_exist( $element ) {
		if ( isset( $element ) && ! empty( $element ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Perform GET TOKEN.
	 *
	 * @param string $login
	 *     The Desk-Net user login.
	 * @param string $password
	 *     The Desk-Net user password.
	 *
	 * @return string
	 */
	public function desk_net_get_token( $login, $password ) {
		/**
		 * @var array request params.
		 */
		$args = array(
			'sslverify' => false,
			'body'      => [
				'grant_type'    => 'client_credentials',
				'client_id'     => $login,
				'client_secret' => $password,
			],
			'method'    => 'POST',
		);

		// The HTTP POST request for get Token.
		$obj_response = wp_remote_post( Desk_Net_Plugin_Settings::DN_BASE_URL
		                                . '/api/token', $args );
		// Detect WP_ERROR.
		if ( is_wp_error( $obj_response ) ) {
			return false;
		}
		$body_response = json_decode( $obj_response['body'], true );

		if ( 200 !== (int) $obj_response['response']['code'] ) {
			update_option( 'wp_desk_net_token', 'not_valid' );

			return false;
		}

		// Update token option.
		update_option( 'wp_desk_net_token', $body_response['access_token'] );

		return $body_response['access_token'];
	}

	/**
	 * Perform HTTP POST/DELETE request.
	 *
	 * @param array  $data
	 *     The upload data.
	 * @param string $url
	 *     The Base API url.
	 * @param string $type
	 *     The Desk-Net API method.
	 * @param string $http_request
	 *     The Custom HTTP request.
	 * @param string $record_id
	 *     ID element.
	 *
	 * @return string
	 */
	public function desk_net_custom_request(
		$data, $url, $type, $http_request, $record_id = ''
	) {
		$request_url = $url . "/api/v1_0_1/{$type}/{$record_id}";

		if ( null !== $data ) {
			$data = wp_json_encode( $data,
				JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES );

			// Delete ampersand encode.
			$data = str_replace( '&amp;', '&', $data );

			$args = array(
				'headers' => [
					'Authorization'  => "bearer $this->token",
					'Content-Length' => strlen( $data ),
					'Content-Type'   => 'application/json;charset=UTF-8',
				],
				'body'    => $data,
			);
		} else {
			$args = array(
				'headers' => [
					'Authorization' => "bearer $this->token",
					'Content-Type'  => 'application/json;charset=UTF-8',
				],
			);
		}

		$args['method'] = $http_request;

		return $this->desk_net_send_request( $request_url, $args, $data );
	}

	/**
	 * Perform HTTP GET request.
	 *
	 * @param string $url
	 *     The Base API url.
	 * @param string $type
	 *     The Desk-Net API method.
	 * @param string $record_id
	 *     The ID element.
	 *
	 * @return string
	 */
	public function desk_net_get( $url, $type, $record_id = '' ) {
		$request_url = $url . "/api/v1_0_1/{$type}/{$record_id}";

		$args = array(
			'headers' => [
				'Authorization' => "bearer $this->token",
			],
			'method'  => 'GET',
		);

		return $this->desk_net_send_request( $request_url, $args );
	}

	/**
	 * Perform HTTP PUT request.
	 *
	 * @param string  $url
	 *     The Base API url.
	 * @param integer $record_id
	 *     The ID element.
	 *
	 * @return string
	 */
	public function desk_net_put( $url, $record_id ) {
		$request_url = $url . "/api/v1_0_1/elements/{$record_id}";

		$args = array(
			'headers' => [
				'Authorization' => "bearer $this->token",
				'Content-Type'  => 'application/json;charset=UTF-8',
			],
			'body'    => $this->update_data,
			'method'  => 'PUT',
		);

		return $this->desk_net_send_request( $request_url, $args );
	}

	/**
	 * Perform check JSON.
	 *
	 * @param string $string
	 *     The line for check.
	 *
	 * @return boolean
	 */
	public function is_json( $string ) {
		json_decode( $string );

		return ( json_last_error() === 0 );
	}

	/**
	 * Perform send HTTP Request.
	 *
	 * @param string  $request_url
	 *     The link for request.
	 * @param array   $args
	 *     The params for request.
	 * @param array   $data
	 *     The post data.
	 * @param boolean $last_request
	 *     The attempt send request.
	 *
	 * @return string|object
	 */
	protected function desk_net_send_request(
		$request_url, $args, $data = null, $last_request = null
	) {
		$args['sslverify'] = false;

		$response = wp_remote_request( $request_url, $args );
		// Detect WP_ERROR.
		if ( is_wp_error( $response ) ) {
			return false;
		}

		$http_code = $response['response']['code'];

		switch ( $http_code ) {
			case 401:
				$response = $this->desk_net_unauthorized( $last_request,
					$request_url, $data );
				break;
			case 400:
				$response
					= $this->desk_net_check_platform_schedule( $response );
				break;
			case 200:
				$response = $response['body'];
				break;
			default:
				$response = $this->desk_net_check_request_response( $data,
					$response );
		}

		return $response;
	}

	/**
	 * Perform check user authorized.
	 *
	 * @param string $last_request
	 *     The value check circle number.
	 * @param string $request_url
	 *     The link for request.
	 * @param array  $data
	 *     The post data.
	 *
	 * @return string|array
	 */
	public function desk_net_unauthorized( $last_request, $request_url, $data
	) {
		$this->token = $this->desk_net_get_token( $this->login,
			$this->password );

		$response = 'desk_net_unauthorized';

		if ( 'update_token' !== $last_request ) {
			$args['headers']['Authorization'] = "bearer $this->token";

			$response = $this->desk_net_send_request( $request_url, $args,
				$data, 'update_token' );
		}

		return $response;
	}

	/**
	 * Perform checking platform schedule in response from request.
	 *
	 * @param object $response
	 *     The request response.
	 *
	 * @return string|array
	 */
	public function desk_net_check_platform_schedule( $response ) {
		$body_response = json_decode( $response['body'], true );
		$response      = $response['body'];

		if ( preg_match( "/^Publication date doesn't match platform schedule/",
			$body_response['message'] )
		) {
			$response = '{"message":"platform_schedule"}';
		}

		return $response;
	}

	/**
	 * Perform check response from request method.
	 *
	 * @param array  $data
	 *     The post data.
	 * @param object $response
	 *     The request response.
	 *
	 * @return string|array
	 */
	public function desk_net_check_request_response( $data, $response ) {
		$data          = json_decode( $data, true );
		$body_response = json_decode( $response['body'], true );
		if ( $data ) {
			$this->check_error_message( $body_response,
				$data['publications'][0]['cms_id'] );
		}

		return $response['body'];
	}

	/**
	 * Perform check Error Message.
	 *
	 * @param array  $message_content
	 *     The list message.
	 * @param string $post_id
	 *     The post ID in WordPress.
	 */
	public function check_error_message( $message_content, $post_id ) {
		if ( preg_match( '/^Page with id \[\d.*\] was not found/',
			$message_content['message'], $category_id )
		) {
			$this->desk_net_update_category_list();
			$post = get_post( $post_id );
			$this->desk_net_update_element( $post_id, $post );
		} elseif ( preg_match( '/^User with id \[\d.*\] was not found/',
			$message_content['message'] )
		) {
			preg_match( '/\d.*\d/', $message_content['message'],
				$desk_net_user_id );

			$args = array(
				'meta_key'   => 'desk_net_author_id',
				'meta_value' => $desk_net_user_id[0],
			);

			$user_data = get_users( $args );

			if ( empty( $user_data[0]->data->ID ) ) {
				$this->error_message = 4;

				return;
			}

			// Detecting WordPress VIP server.
			if ( defined( 'WPCOM_IS_VIP_ENV' ) && true === WPCOM_IS_VIP_ENV ) {
				delete_user_attribute( $user_data[0]->data->ID,
					'desk_net_author_id', $desk_net_user_id[0] );
			} else {
				// @codingStandardsIgnoreStart.
				delete_user_meta( $user_data[0]->data->ID, 'desk_net_author_id',
					$desk_net_user_id[0] );
				// @codingStandardsIgnoreEnd.
			}

			$post = get_post( $post_id );
			$this->desk_net_update_element( $post_id, $post );
		} else {
			$this->error_message = 4;
		}
	}
}

new Desk_Net_Request_Methods();
