<?php

/**
 * @package Desk_Net_Update_Methods.
 * @author  Desk-Net GmbH.
 */

include_once plugin_dir_path( __FILE__ ) . 'class-desk-net-request-methods.php';
include_once plugin_dir_path( __FILE__ ) . 'class-desk-net-plugin-settings.php';

class Desk_Net_Update_Methods extends Desk_Net_Request_Methods {
	/**
	 * Perform updating story.
	 *
	 * @param integer $post_id
	 *     The post id in WP.
	 * @param object  $post
	 *     The post object in WP.
	 * @param object  $post_revision
	 *     The post object before update.
	 *
	 * @return boolean
	 */
	public function desk_net_update_story( $post_id, $post, $post_revision ) {
		// Default value.
		$post_status_update   = false;
		$post_category_update = false;
		// Getting Desk-Net story id.
		$story_id             = get_post_meta( $post_id, 'story_id', true );
		$removed_status_on_dn = get_post_meta( $post_id,
			'wp_desk_net_remove_status', true );

		// Checking update story on Desk-Net.
		if ( 'removed' === $removed_status_on_dn ) {
			// Hide admin Notice.
			$this->error_message = 0;

			return false;
		} elseif ( empty( $story_id ) ) {
			$this->desk_net_create_story( $post_id, $post );

			return false;
		}

		if ( 'publish' == get_post_meta( $post_id,
				'wp_desk_net_post_previous_status', true )
		     && 'publish' == $post->post_status
		) {
			// Hide admin Notice.
			$this->error_message = 0;
			$this->set_desk_net_post_notice( $post_id, $this->error_message );

			return false;
		}

		// Get JSON with default value.
		$story_data
			= $this->desk_net_get( Desk_Net_Plugin_Settings::DN_BASE_URL,
			'elements', $story_id );

		$story_data = json_decode( $story_data, true );

		if ( ! $this->desk_net_is_valid_json_data( $story_data, $story_id ) ) {
			$this->set_desk_net_post_notice( $post_id, $this->error_message );

			return false;
		}

		// Checking post status.
		if ( ! is_object( $post_revision ) || ! is_object( $post )
		     || $post_revision->post_status !== $post->post_status
		) {
			$post_status_update = true;
		}
		// Getting first active category.
		$selected_categories = get_the_terms( $post_id, 'category' );
		// Default value if not selected categories.
		$active_category = 'not_active_category';

		if ( ! empty( $selected_categories ) ) {
			// Setting active post category to post revision.
			$active_category = array_values( $selected_categories )[0]->term_id;
		}

		// Checking post category.
		if ( (int) get_post_meta( $post_id, 'wp_desk_net_category_revision',
				true ) !== (int) $active_category
		     || wp_json_encode( get_post_meta( $post_id,
				'wp_desk_net_selected_categories_revision', true ) )
		        !== wp_json_encode( $selected_categories )
		) {
			$post_category_update = true;
			update_post_meta( $post_id,
				'wp_desk_net_selected_categories_revision',
				get_the_terms( $post_id, 'category' ) );
			update_post_meta( $post_id, 'wp_desk_net_category_revision',
				$active_category );
		}

		// Update JSON data before send to Desk-Net.
		$story_data = $this->update_json_data( $story_data, $post_id,
			$post_status_update, $post_category_update );

		// Update with error.
		if ( ! $story_data ) {
			$this->error_message = 4;
			$this->set_desk_net_post_notice( $post_id, $this->error_message );

			return false;
		}

		$this->update_data = wp_json_encode( $story_data,
			JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES );
		// Encode Ampersand.
		$this->update_data = str_replace( '&amp;', '&', $this->update_data );
		$this->update_data = apply_filters( 'wp_desk_net_update_story',
			$this->update_data );
		// Updating Desk-Net element.
		$response = $this->desk_net_put( Desk_Net_Plugin_Settings::DN_BASE_URL,
			$story_data['id'] );
		$response = json_decode( $response, true );

		// Convert response to array.
		if ( is_object( $response ) ) {
			$response = json_decode( $response->data, true );
		}

		if ( ! empty( $response['message'] ) ) {
			$this->error_message = ( 'platform_schedule'
			                         === $response['message'] ) ? 8 : 4;
			$this->set_desk_net_post_notice( $post_id, $this->error_message );

			return false;
		}

		$this->set_published_story_date_in_post_date( $response, $post_id );
		$desk_net_status_name = $this->get_response_status( $response,
			$post_id );

		// Updating Desk-Net status for Post.
		if ( $desk_net_status_name ) {
			update_post_meta( $post_id, 'desk_net_status_name',
				$desk_net_status_name );
			update_post_meta( $post_id, 'desk_net_status_id',
				$response['publications'][0]['status'] );
		}

		update_post_meta( $post_id, 'wp_desk_net_post_previous_status',
			$post->post_status );

		$this->set_desk_net_post_notice( $post_id, $this->error_message );

		return true;
	}

	/**
	 * Validating response data.
	 *
	 * @param array   $story_data
	 *   The story data.
	 * @param integer $story_id
	 *   The story id.
	 *
	 * @return bool
	 */
	private function desk_net_is_valid_json_data( $story_data, $story_id ) {
		if ( empty( $story_data ) ) {
			$this->error_message = 3;
		}

		if ( ! empty( $story_data['message'] ) ) {
			$this->error_message = str_contains( $story_data['message'],
				'Element with id [' . $story_id . '] was not found' ) ? 2 : 3;
		}

		return ! in_array( $this->error_message, [ 2, 3 ] );
	}

	/**
	 * Creating Story in Desk-Net.
	 *
	 * @param integer $post_id
	 *   The post id in WP.
	 * @param object  $post
	 *   The post.
	 *
	 * @return bool
	 */
	public function desk_net_create_story( $post_id, $post ) {
		$post_data = [];

		$post_data['title'] = $post->post_title;
		// Updating Slug value.
		$post_data = $this->send_slug_by_CMS( $post, $post_data );

		$platform_id          = get_option( 'wp_desk_net_platform_id' );
		$publication_position = 0;
		$publications_id      = get_post_meta( $post_id, 'publications_id',
			true );
		$post_permalink       = ( get_permalink( $post_id ) )
			? get_permalink( $post_id ) : $post->guid;

		if ( ! empty( $publications_id ) ) {
			foreach ( $post_data['publications'] as $key => $value ) {
				if ( (int) $post_data['publications'][ $key ]['id']
				     === (int) $publications_id
				) {
					$publication_position = $key;
					break;
				}
			}
		}

		$publication =& $post_data['publications'][ $publication_position ];

		$publication['url_to_content_in_cms']    = get_edit_post_link( $post_id );
		$publication['url_to_published_content'] = $post_permalink;
		$publication['assignments']              = [ true ];

		// Updating Date and time fields.
		if ( ! empty( $post_date_gmt )
		     || '0000-00-00 00:00:00' !== $post->post_date_gmt
		) {
			$publication = $this->set_date_and_time_to_story( $publication,
				$post_id, $post );
		}

		// Setting Category.
		$post_data = $this->desk_net_category_mapping_wp_to_dn( $post_id,
			$post_data, $publication_position, $platform_id );
		// Check category on status 'Do not import'.
		if ( 'Do not import' === $post_data['publications'] ) {
			// Hide admin Notice.
			$this->error_message = 0;
			$this->set_desk_net_post_notice( $post_id, $this->error_message );

			return false;
		}

		// Detecting WordPress VIP server.
		if ( defined( 'WPCOM_IS_VIP_ENV' ) && true === WPCOM_IS_VIP_ENV ) {
			// Getting Author info.
			$author_id = get_user_attribute( get_post_field( 'post_author',
				$post_id ), 'desk_net_author_id' );
		} else {
			// Getting Author info.
			// @codingStandardsIgnoreStart.
			$author_id = get_user_meta( get_post_field( 'post_author',
				$post_id ), 'desk_net_author_id', true );
			// @codingStandardsIgnoreEnd.
		}

		if ( ! empty( $author_id ) ) {
			$post_data['tasks'][0]['user'] = intval( $author_id );
		} else {
			$user = get_userdata( $post->post_author );

			$post_data['tasks'][0]['user']['name']  = $user->data->display_name;
			$post_data['tasks'][0]['user']['email'] = $user->data->user_email;
		}

		//Default values.
		$post_data['tasks'][0]['format']             = 18;
		$post_data['tasks'][0]['confirmationStatus'] = - 2;

		$publication['cms_id'] = $post_id;

		// Updating Status.
		$post_status = ( $this->is_scheduled_status( $post_id ) ) ? 'scheduled'
			: get_post_status( $post_id );
		$status_id   = $this->status_matching( $post_status );

		if ( ! empty( $status_id ) && 'No Status' !== $status_id ) {
			$publication['status'] = $status_id;

			$post_data['tasks'][0]['status'] = 1;
		} elseif ( 'No Status' === $status_id ) {
			$post_data['tasks'][0]['status'] = 1;

			$publication['status'] = 1;
		}

		$post_data = apply_filters( 'wp_desk_net_insert_story', $post_data );
		//Creating new element on Desk-Net.
		$response = $this->desk_net_custom_request( $post_data,
			Desk_Net_Plugin_Settings::DN_BASE_URL, 'elements', 'POST' );

		if ( 'desk_net_unauthorized' == $response ) {
			$this->error_message = 5;
		} else if ( 1 === $this->error_message ) {
			$response = json_decode( $response, true );

			if ( ! empty( $response['message'] ) ) {
				if ( 'platform_schedule' === $response['message'] ) {
					$this->error_message = 7;
				} else {
					$this->error_message = 6;
				}
			} else {
				$this->set_published_story_date_in_post_date( $response,
					$post_id );
				$desk_net_status_name = $this->get_response_status( $response,
					$post_id );

				// Setting new Desk-Net data.
				update_post_meta( $post_id, 'desk_net_description',
					html_entity_decode( mb_convert_encoding( $response['title'],
						'UTF-8' ) ) );
				update_post_meta( $post_id, 'story_id', $response['id'] );
				update_post_meta( $post_id, 'desk_net_status_name',
					$desk_net_status_name );
				update_post_meta( $post_id, 'publications_id',
					$response['publications'][0]['id'] );

				if ( $desk_net_status_name ) {
					update_post_meta( $post_id, 'desk_net_status_id',
						$response['publications'][0]['status'] );
				}
			}
		}

		$this->set_desk_net_post_notice( $post_id, $this->error_message );

		add_filter( 'redirect_post_location', array(
			$this,
			'add_notice_query_var',
		), 99 );

		return true;
	}

	/**
	 * Getting story status after creating/updating the story in Desk-Net.
	 *
	 * @param array $response
	 *   The response data after creating/updating the story in Desk-Net.
	 * @param int   $post_id
	 *   The post ID.
	 *
	 * @return string
	 *   Story status name on Desk-Net side.
	 */
	public function get_response_status( $response, $post_id ) {
		$publication_position = $this->get_publication_position( $response,
			$post_id );

		if ( isset( $response['publications'][ $publication_position ]['status'] )
		     && ! empty( $response['publications'][ $publication_position ]['status'] )
		) {
			$desk_net_status_name
				= $this->desk_net_get_status_name_by_id( $response['publications'][ $publication_position ]['status'] );
		} else {
			$desk_net_status_name
				= $this->desk_net_get_status_name_by_id( $response['tasks'][ $publication_position ]['status'] );
		}

		return $desk_net_status_name;
	}

	/**
	 * Setting the Story date to the Post date when the story was published in Desk-Net.
	 *
	 * @param array $response
	 *   The response data after creating/updating the story in Desk-Net.
	 * @param int   $post_id
	 *   The post ID.
	 */
	public function set_published_story_date_in_post_date( $response, $post_id
	) {
		$publication_position = $this->get_publication_position( $response,
			$post_id );

		$publication =& $response['publications'][ $publication_position ];

		if ( isset( $publication['status'] ) ) {
			$story_status = $publication['status'];

			if ( 5 === $story_status ) {
				$date = $publication['published']['start'];
				$date = $this->get_story_date( $date );

				if ( $date ) {
					$postarr = [ 'ID' => $post_id ];
					$postarr = $this->set_post_date( $postarr, $date );

					wp_update_post( $postarr );
				}
			}
		}
	}

	/**
	 * Setting date field in JSON.
	 *
	 * @param array   $publication
	 *   The publication data.
	 * @param integer $post_id
	 *   The post id in WP.
	 * @param object  $post
	 *   The post.
	 *
	 * @return array
	 *   Return publication.
	 */
	private function set_date_and_time_to_story( $publication, $post_id, $post
	) {
		$date = strtotime( $post->post_date_gmt );

		if ( ! empty( $publication['recurring'] ) ) {
			if ( $this->desk_net_is_publish_time_exist( $date ) ) {
				$publication['recurring']['time'] = date( 'H:i', $date );
			}

			$publication['recurring']['start']
				= $this->desk_net_get_post_date( $post_id,
				$post );
		} else {
			if ( $this->desk_net_is_publish_time_exist( $date ) ) {
				$publication['single']['start']['time'] = date( 'H:i', $date );
			}

			$publication['single']['start']['date']
				= $this->desk_net_get_post_date( $post_id,
				$post );
		}

		return $publication;
	}

	/**
	 * The updating slug in Desk-Net from CMS.
	 *
	 * @param integer $post_id
	 *   The post id in WP.
	 * @param object  $post
	 *   The post.
	 *
	 * @return string
	 *   Return post date.
	 */
	private function desk_net_get_post_date( $post_id, $post ) {
		$date = strtotime( $post->post_date_gmt );
		delete_post_meta( $post_id, 'wp_desk_net_change_post_date',
			'user_changed_post_date' );

		return date( 'Y-m-d', $date );
	}


	/**
	 * Checking the Post date on 23:59 value.
	 *
	 * @param int $date
	 *   The post date.
	 *
	 * @return bool
	 */
	private function desk_net_is_publish_time_exist( $date ) {
		// Adding timezone.
		$date = $date + get_option( 'gmt_offset' ) * 60 * 60;

		return date( 'H:i', $date ) !== '23:59';
	}

	/**
	 * The updating slug in Desk-Net from CMS.
	 *
	 * @param object $post
	 *   The post object.
	 * @param array  $post_data
	 *   The story data.
	 *
	 * @return array
	 *   Return updated story data.
	 */
	public function send_slug_by_CMS( $post, $post_data ) {
		if ( $this->is_slug_syncing( $post->ID ) ) {
			// Getting Slug setting value.
			$slug_syncing = get_option( 'wp_desk_net_slug_slug_syncing' );

			switch ( $slug_syncing ) {
				case 'title':
					$post_data['slug'] = $post->post_title;

					break;

				default:
					if ( empty( $post->post_name ) ) {
						$post_data['slug']
							= sanitize_title( $post->post_title );
					} else {
						$post_data['slug'] = $post->post_name;
					}
			}
		}

		return $post_data;
	}

	/**
	 * Updating JSON Status Matching data.
	 *
	 * @param int   $post_id
	 *     The post_id - which was updated.
	 * @param array $story_data
	 *     The new post data.
	 * @param int   $publication_position
	 *     The publication position number.
	 * @param int   $platform_id
	 *     The platform_id - which was updated.
	 *
	 * @return array
	 *   The new data with updated category id.
	 */
	public function desk_net_category_mapping_wp_to_dn(
		$post_id, array $story_data, $publication_position, $platform_id
	) {
		$select_category = get_the_terms( $post_id, 'category' );

		if ( ! empty( $select_category ) ) {
			// Clear category value.
			$category_value = 'Do not import';
			// Scan category mapping WP to Desk-Net.
			foreach ( $select_category as $cat ) {
				$category_key          = 'wp_desk_net_category_wp_to_desk_net_'
				                         . $cat->term_id;
				$desk_net_category_key = get_option( $category_key );
				// Checking category mapping on exist value.
				if ( false === $desk_net_category_key
				     && 'Do not import' === $category_value
				) {
					$category_value = 'No category';
					continue;

				}

				if ( 'Do not import' !== $category_value
				     && 'No category' !== $category_value
				) {
					break;

				} elseif ( ! empty( $desk_net_category_key )
				           && 'Do not import' !== $desk_net_category_key
				) {
					$category_value = $desk_net_category_key;
				}
			}
			// Set Desk-Net category.
			switch ( $category_value ) {
				case 'Do not import':
					$story_data['publications'] = 'Do not import';
					break;

				case 'No category':
					unset( $story_data['publications'][ $publication_position ]['category'] );
					$story_data['publications'][ $publication_position ]['platform']
						= $platform_id;
					break;

				default:
					unset( $story_data['publications'][ $publication_position ]['platform'] );
					$story_data['publications'][ $publication_position ]['category']
						= (int) $category_value;
			}
		} else {
			// Default mapping on 'No category'.
			unset( $story_data['publications'][ $publication_position ]['category'] );
			$story_data['publications'][ $publication_position ]['platform']
				= $platform_id;
		}

		return $story_data;
	}

	/**
	 * Perform status Matching WP to Desk-Net.
	 *
	 * @param string $wp_status
	 *     The WP post Status.
	 *
	 * @return string
	 */
	public function status_matching( $wp_status ) {
		$desk_net_status
			= get_option( "wp_desk_net_status_wp_to_desk_net_$wp_status" );

		if ( ! empty( $desk_net_status ) && 'No Status' !== $desk_net_status ) {
			return $desk_net_status;
		} else {
			return 'No Status';
		}
	}

	/**
	 * Checking current post on the scheduled status.
	 *
	 * @param integer $post_id
	 *   Post ID
	 *
	 * @return bool
	 */
	public function is_scheduled_status( $post_id ) {
		$post = get_post( $post_id );

		return 'future' === $post->post_status;
	}

	/**
	 * Perform get Desk-Net status name by ID
	 *
	 * @param integer $status_id
	 *   The Desk-Net Status ID
	 *
	 * @return string
	 */
	public function desk_net_get_status_name_by_id( $status_id ) {
		$keys_status_field = [
			'wp_desk_net_desk-net-list-active-status',
			'wp_desk_net_desk-net-list-deactivated-status',
		];

		foreach ( $keys_status_field as $value ) {
			$status_list = get_option( $value );

			if ( false !== $status_list ) {
				foreach ( $status_list as $key => $status ) {
					if ( (int) $status_list[ $key ]['id']
					     === (int) $status_id
					) {
						return $status_list[ $key ]['name'];
					}
				}
			}
		}

		return __( 'No status', 'desk_net' );
	}

	/**
	 * Updating JSON Data before send to Desk-Net.
	 *
	 * @param array $story_data
	 *     The new post data.
	 * @param int   $post_id
	 *     The post id in WP.
	 * @param bool  $post_status_update
	 *     The status updating post status.
	 * @param bool  $post_category_update
	 *     The status updating post category.
	 *
	 * @return array|bool
	 */
	public function update_json_data(
		$story_data, $post_id, $post_status_update, $post_category_update
	) {
		// Getting post data.
		$post                 = get_post( $post_id );
		$platform_id          = get_option( 'wp_desk_net_platform_id' );
		$post_permalink       = ( get_permalink( $post_id ) )
			? get_permalink( $post_id ) : $post->guid;
		$publication_position = $this->get_publication_position( $story_data,
			$post_id );

		// Updating Slug value.
		$story_data = $this->send_slug_by_CMS( $post, $story_data );

		// Updating elements.
		$publication =& $story_data['publications'][ $publication_position ];

		$publication['url_to_content_in_cms']    = get_edit_post_link( $post_id );
		$publication['url_to_published_content'] = $post_permalink;

		// Updating Date field.
		if ( ! empty( $post_date_gmt )
		     || '0000-00-00 00:00:00' !== $post->post_date_gmt
		) {
			$publication = $this->set_date_and_time_to_story( $publication,
				$post_id, $post );
		}

		if ( ! empty( $story_data['tasks'] ) ) {
			// Detecting WordPress VIP server.
			if ( defined( 'WPCOM_IS_VIP_ENV' ) && true === WPCOM_IS_VIP_ENV ) {
				//Getting Author info.
				$author_id = get_user_attribute( get_post_field( 'post_author',
					$post_id ), 'desk_net_author_id' );
			} else {
				// Update Author.
				// @codingStandardsIgnoreStart.
				$author_id = get_user_meta( get_post_field( 'post_author',
					$post_id ), 'desk_net_author_id', true );
				// @codingStandardsIgnoreEnd.
			}

			if ( ! empty( $author_id ) ) {
				$story_data['tasks'][0]['user'] = intval( $author_id );
			}
			// Update task status.
			$story_data['tasks'][0]['status'] = 1;
		}

		// Updating Status filed.
		if ( $post_status_update ) {
			$post_status = ( $this->is_scheduled_status( $post_id ) )
				? 'scheduled'
				: get_post_status( $post_id );
			$status_id   = $this->status_matching( $post_status );

			if ( ! empty( $status_id ) && 'No Status' !== $status_id ) {
				$publication['status']
					= $status_id;
			} elseif ( 'No Status' === $status_id ) {
				$publication['status']
					= 1;
			}
		}

		// Updating Category field.
		if ( $post_category_update ) {
			$story_data = $this->desk_net_category_mapping_wp_to_dn( $post_id,
				$story_data, $publication_position, $platform_id );
			// Check category on status 'Do not import'.
			if ( 'Do not import' === $story_data['publications'] ) {
				return false;
			}
		}

		if ( empty( $story_data['id'] ) ) {
			$this->error_message = 3;
			$this->set_desk_net_post_notice( $post_id, $this->error_message );

			return false;
		}

		return $story_data;
	}

	/**
	 * Getting publication position in JSON Data.
	 *
	 * @param array $story_data
	 *     The story data.
	 * @param int   $post_id
	 *     The post id in WP.
	 *
	 * @return int
	 */
	public function get_publication_position( $story_data, $post_id ) {
		$publication_position = 0;
		$publications_id      = get_post_meta( $post_id, 'publications_id' );

		if ( ! empty( $publications_id ) ) {
			foreach ( (array) $story_data['publications'] as $key => $value ) {
				if ( (int) $story_data['publications'][ $key ]['id']
				     === (int) $publications_id
				) {
					$publication_position = $key;
					break;
				}
			}
		}

		return $publication_position;
	}
}
